@@ -10,7 +10,9 @@ datacube is licensed under the Apache License, Version 2.0:
 
 """
 import copy
+import importlib.metadata
 import os
+from pathlib import Path
 from typing import (
     Any,
     Dict,
@@ -57,10 +59,14 @@ from rioxarray.rioxarray import (
     _order_bounds,
 )
 
+_RASTERIO_GTE_12 = version.parse(
+    importlib.metadata.version("rasterio")
+) >= version.parse("1.2")
+
 # DTYPE TO NODATA MAP
 # Based on: https://github.com/OSGeo/gdal/blob/
-# cde27dc7641964a872efdc6bbcf5e3d3f7ab9cfd/gdal/
-# swig/python/gdal-utils/osgeo_utils/gdal_calc.py#L62
+# dee861e7c91c2da7ef8ff849947713e4d9bd115c/
+# swig/python/gdal-utils/osgeo_utils/gdal_calc.py#L61
 _NODATA_DTYPE_MAP = {
     1: 255,  # GDT_Byte
     2: 65535,  # GDT_UInt16
@@ -69,10 +75,13 @@ _NODATA_DTYPE_MAP = {
     5: -2147483647,  # GDT_Int32
     6: 3.402823466e38,  # GDT_Float32
     7: 1.7976931348623158e308,  # GDT_Float64
-    8: -32768,  # GDT_CInt16
-    9: -2147483647,  # GDT_CInt32
+    8: None,  # GDT_CInt16
+    9: None,  # GDT_CInt32
     10: 3.402823466e38,  # GDT_CFloat32
     11: 1.7976931348623158e308,  # GDT_CFloat64
+    12: None,  # GDT_Int64
+    13: None,  # GDT_UInt64
+    14: None,  # GDT_Int8
 }
 
 
@@ -504,7 +513,7 @@ class RasterArray(XRasterBase):
 
     def _get_dst_nodata(self, nodata: Optional[float]) -> Optional[float]:
         default_nodata = (
-            _NODATA_DTYPE_MAP[dtype_rev[self._obj.dtype.name]]
+            _NODATA_DTYPE_MAP.get(dtype_rev[self._obj.dtype.name])
             if self.nodata is None
             else self.nodata
         )
@@ -719,7 +728,7 @@ class RasterArray(XRasterBase):
         maxy: float,
         auto_expand: Union[bool, int] = False,
         auto_expand_limit: int = 3,
-        crs: Any = None,
+        crs: Optional[Any] = None,
     ) -> xarray.DataArray:
         """Clip the :obj:`xarray.DataArray` by a bounding box.
 
@@ -840,7 +849,7 @@ class RasterArray(XRasterBase):
     def clip(
         self,
         geometries: Iterable,
-        crs: Any = None,
+        crs: Optional[Any] = None,
         all_touched: bool = False,
         drop: bool = True,
         invert: bool = False,
@@ -904,7 +913,7 @@ class RasterArray(XRasterBase):
             )
         crs = crs_from_user_input(crs) if crs is not None else self.crs
         if self.crs != crs:
-            if version.parse(rasterio.__version__) >= version.parse("1.2"):
+            if _RASTERIO_GTE_12:
                 geometries = rasterio.warp.transform_geom(crs, self.crs, geometries)
             else:
                 geometries = [
@@ -1046,9 +1055,9 @@ class RasterArray(XRasterBase):
     def to_raster(
         self,
         raster_path: Union[str, os.PathLike],
-        driver: str = None,
-        dtype: Union[str, np.dtype] = None,
-        tags: Dict[str, str] = None,
+        driver: Optional[str] = None,
+        dtype: Optional[Union[str, np.dtype]] = None,
+        tags: Optional[Dict[str, str]] = None,
         windowed: bool = False,
         recalc_transform: bool = True,
         lock: Optional[bool] = None,
@@ -1101,10 +1110,12 @@ class RasterArray(XRasterBase):
             is True. Otherwise None is returned.
 
         """
-        if driver is None and version.parse(rasterio.__version__) < version.parse(
-            "1.2"
-        ):
-            driver = "GTiff"
+        if driver is None:
+            extension = Path(raster_path).suffix
+            # https://github.com/rasterio/rasterio/pull/2634
+            # https://github.com/rasterio/rasterio/pull/2008
+            if not _RASTERIO_GTE_12 or extension in (".tif", ".tiff"):
+                driver = "GTiff"
 
         # get the output profile from the rasterio object
         # if opened with xarray.open_rasterio()
