@@ -252,7 +252,7 @@ class RasterDataset(XRasterBase):
         maxy: float,
         auto_expand: Union[bool, int] = False,
         auto_expand_limit: int = 3,
-        crs: Any = None,
+        crs: Optional[Any] = None,
     ) -> xarray.Dataset:
         """Clip the :class:`xarray.Dataset` by a bounding box in dimensions 'x'/'y'.
 
@@ -314,7 +314,7 @@ class RasterDataset(XRasterBase):
     def clip(
         self,
         geometries: Iterable,
-        crs: Any = None,
+        crs: Optional[Any] = None,
         all_touched: bool = False,
         drop: bool = True,
         invert: bool = False,
@@ -440,9 +440,9 @@ class RasterDataset(XRasterBase):
     def to_raster(
         self,
         raster_path: Union[str, os.PathLike],
-        driver: str = None,
-        dtype: Union[str, np.dtype] = None,
-        tags: Dict[str, str] = None,
+        driver: Optional[str] = None,
+        dtype: Optional[Union[str, np.dtype]] = None,
+        tags: Optional[Dict[str, str]] = None,
         windowed: bool = False,
         recalc_transform: bool = True,
         lock: Optional[bool] = None,
@@ -497,18 +497,23 @@ class RasterDataset(XRasterBase):
         """
         variable_dim = f"band_{uuid4()}"
         data_array = self._obj.to_array(dim=variable_dim)
-        # write data array names to raster
-        data_array.attrs["long_name"] = data_array[variable_dim].values.tolist()
         # ensure raster metadata preserved
         scales = []
         offsets = []
         nodatavals = []
+        band_tags = []
+        long_name = []
         for data_var in data_array[variable_dim].values:
             scales.append(self._obj[data_var].attrs.get("scale_factor", 1.0))
             offsets.append(self._obj[data_var].attrs.get("add_offset", 0.0))
+            long_name.append(self._obj[data_var].attrs.get("long_name", data_var))
             nodatavals.append(self._obj[data_var].rio.nodata)
+            band_tags.append(self._obj[data_var].attrs.copy())
         data_array.attrs["scales"] = scales
         data_array.attrs["offsets"] = offsets
+        data_array.attrs["band_tags"] = band_tags
+        data_array.attrs["long_name"] = long_name
+
         nodata = nodatavals[0]
         if (
             all(nodataval == nodata for nodataval in nodatavals)
