@@ -15,6 +15,7 @@ import rasterio
 from rasterio.windows import Window
 from xarray.conventions import encode_cf_variable
 
+from rioxarray._io import _get_unsigned_dtype
 from rioxarray.exceptions import RioXarrayError
 
 try:
@@ -35,28 +36,10 @@ UNWANTED_RIO_ATTRS = ("nodatavals", "is_tiled", "res")
 # Note: transform & crs are removed in write_transform/write_crs
 
 
-def _write_metatata_to_raster(raster_handle, xarray_dataset, tags):
+def _write_tags(raster_handle, tags):
     """
-    Write the metadata stored in the xarray object to raster metadata
+    Write tags to raster dataset
     """
-    tags = xarray_dataset.attrs if tags is None else {**xarray_dataset.attrs, **tags}
-
-    # write scales and offsets
-    try:
-        raster_handle.scales = tags["scales"]
-    except KeyError:
-        scale_factor = tags.get(
-            "scale_factor", xarray_dataset.encoding.get("scale_factor")
-        )
-        if scale_factor is not None:
-            raster_handle.scales = (scale_factor,) * raster_handle.count
-    try:
-        raster_handle.offsets = tags["offsets"]
-    except KeyError:
-        add_offset = tags.get("add_offset", xarray_dataset.encoding.get("add_offset"))
-        if add_offset is not None:
-            raster_handle.offsets = (add_offset,) * raster_handle.count
-
     # filter out attributes that should be written in a different location
     skip_tags = (
         UNWANTED_RIO_ATTRS
@@ -75,10 +58,19 @@ def _write_metatata_to_raster(raster_handle, xarray_dataset, tags):
     # in this case, it will be stored in the raster description
     if not isinstance(tags.get("long_name"), str):
         skip_tags += ("long_name",)
+    band_tags = tags.pop("band_tags", [])
     tags = {key: value for key, value in tags.items() if key not in skip_tags}
     raster_handle.update_tags(**tags)
 
-    # write band name information
+    if isinstance(band_tags, list):
+        for iii, band_tag in enumerate(band_tags):
+            raster_handle.update_tags(iii + 1, **band_tag)
+
+
+def _write_band_description(raster_handle, xarray_dataset):
+    """
+    Write band descriptions using the long name
+    """
     long_name = xarray_dataset.attrs.get("long_name")
     if isinstance(long_name, (tuple, list)):
         if len(long_name) != raster_handle.count:
@@ -95,6 +87,36 @@ def _write_metatata_to_raster(raster_handle, xarray_dataset, tags):
                 raster_handle.set_band_description(iii + 1, band_description)
 
 
+def _write_metatata_to_raster(raster_handle, xarray_dataset, tags):
+    """
+    Write the metadata stored in the xarray object to raster metadata
+    """
+    tags = (
+        xarray_dataset.attrs.copy()
+        if tags is None
+        else {**xarray_dataset.attrs, **tags}
+    )
+
+    # write scales and offsets
+    try:
+        raster_handle.scales = tags["scales"]
+    except KeyError:
+        scale_factor = tags.get(
+            "scale_factor", xarray_dataset.encoding.get("scale_factor")
+        )
+        if scale_factor is not None:
+            raster_handle.scales = (scale_factor,) * raster_handle.count
+    try:
+        raster_handle.offsets = tags["offsets"]
+    except KeyError:
+        add_offset = tags.get("add_offset", xarray_dataset.encoding.get("add_offset"))
+        if add_offset is not None:
+            raster_handle.offsets = (add_offset,) * raster_handle.count
+
+    _write_tags(raster_handle=raster_handle, tags=tags)
+    _write_band_description(raster_handle=raster_handle, xarray_dataset=xarray_dataset)
+
+
 def _ensure_nodata_dtype(original_nodata, new_dtype):
     """
     Convert the nodata to the new datatype and raise warning
@@ -224,11 +246,25 @@ class RasterioWriter:
         **kwargs
             Keyword arguments to pass into writing the raster.
         """
+        xarray_dataarray = xarray_dataarray.copy()
         kwargs["dtype"], numpy_dtype = _get_dtypes(
             kwargs["dtype"],
             xarray_dataarray.encoding.get("rasterio_dtype"),
             xarray_dataarray.encoding.get("dtype", str(xarray_dataarray.dtype)),
         )
+        # there is no equivalent for netCDF _Unsigned
+        # across output GDAL formats. It is safest to convert beforehand.
+        # https://github.com/OSGeo/gdal/issues/6352#issuecomment-1245981837
+        if "_Unsigned" in xarray_dataarray.encoding:
+            unsigned_dtype = _get_unsigned_dtype(
+                unsigned=xarray_dataarray.encoding["_Unsigned"] == "true",
+                dtype=numpy_dtype,
+            )
+            if unsigned_dtype is not None:
+                numpy_dtype = unsigned_dtype
+                kwargs["dtype"] = unsigned_dtype
+                xarray_dataarray.encoding["rasterio_dtype"] = str(unsigned_dtype)
+                xarray_dataarray.encoding["dtype"] = str(unsigned_dtype)
 
         if kwargs["nodata"] is not None:
             # Ensure dtype of output data matches the expected dtype.
