@@ -435,8 +435,8 @@ class XRasterBase:
 
     def write_crs(
         self,
-        input_crs: Any = None,
-        grid_mapping_name: str = None,
+        input_crs: Optional[Any] = None,
+        grid_mapping_name: Optional[str] = None,
         inplace: bool = False,
     ) -> Union[xarray.Dataset, xarray.DataArray]:
         """
@@ -489,10 +489,12 @@ class XRasterBase:
             )
         # add grid mapping coordinate
         data_obj.coords[grid_mapping_name] = xarray.Variable((), 0)
+        grid_map_attrs = {}
         if get_option(EXPORT_GRID_MAPPING):
-            grid_map_attrs = pyproj.CRS.from_user_input(data_obj.rio.crs).to_cf()
-        else:
-            grid_map_attrs = {}
+            try:
+                grid_map_attrs = pyproj.CRS.from_user_input(data_obj.rio.crs).to_cf()
+            except KeyError:
+                pass
         # spatial_ref is for compatibility with GDAL
         crs_wkt = data_obj.rio.crs.to_wkt()
         grid_map_attrs["spatial_ref"] = crs_wkt
@@ -1195,7 +1197,7 @@ class XRasterBase:
         self,
         gcps: Iterable[GroundControlPoint],
         gcp_crs: Any,
-        grid_mapping_name: str = None,
+        grid_mapping_name: Optional[str] = None,
         inplace: bool = False,
     ) -> Union[xarray.Dataset, xarray.DataArray]:
         """
