@@ -23,13 +23,16 @@ from rasterio.errors import NotGeoreferencedWarning
 from rasterio.transform import from_origin
 from rasterio.warp import calculate_default_transform
 from xarray import DataArray
-from xarray.coding.variables import SerializationWarning
 from xarray.testing import assert_allclose, assert_equal, assert_identical
 
 import rioxarray
 from rioxarray._io import build_subdataset_filter
 from rioxarray.rioxarray import DEFAULT_GRID_MAP
 from test.conftest import (
+    GDAL_GE_36,
+    RASTERIO_GE_13,
+    RASTERIO_GE_125,
+    RASTERIO_VERSION,
     TEST_COMPARE_DATA_DIR,
     TEST_INPUT_DATA_DIR,
     _assert_xarrays_equal,
@@ -40,12 +43,12 @@ from test.integration.test_integration_rioxarray import (
 )
 
 cint_skip = pytest.mark.skipif(
-    rasterio.__version__ < "1.2.4",
+    RASTERIO_VERSION < version.parse("1.2.4"),
     reason="https://github.com/mapbox/rasterio/issues/2182",
 )
 
 python_vsi_skip = pytest.mark.skipif(
-    rasterio.__version__ < "1.3.0",
+    not RASTERIO_GE_13,
     reason="Python VSI Support added in 1.3.0",
 )
 
@@ -220,10 +223,11 @@ def test_open_multiple_resolution():
     )
     assert isinstance(rds_list, list)
     assert len(rds_list) == 2
-    for rds in rds_list:
-        assert rds.attrs["SHORTNAME"] == "MOD09GA"
     assert rds_list[0].dims == {"y": 1200, "x": 1200, "band": 1}
     assert rds_list[1].dims == {"y": 2400, "x": 2400, "band": 1}
+    for rds in rds_list:
+        assert rds.attrs["SHORTNAME"] == "MOD09GA"
+        rds.close()
 
 
 def test_open_group_filter(open_rasterio):
@@ -258,11 +262,16 @@ def test_open_group_load_attrs(open_rasterio):
     ) as rds:
         attrs = rds["sur_refl_b05_1"].attrs
         assert sorted(attrs) == [
+            "Nadir Data Resolution",
             "_FillValue",
             "add_offset",
+            "add_offset_err",
+            "calibrated_nt",
             "long_name",
             "scale_factor",
+            "scale_factor_err",
             "units",
+            "valid_range",
         ]
         assert attrs["long_name"] == "500m Surface Reflectance Band 5 - first layer"
         assert attrs["units"] == "reflectance"
@@ -299,6 +308,7 @@ def test_open_rasterio_mask_chunk_clip():
             (3.0, 0.0, 425047.68381405267, 0.0, -3.0, 4615780.040546387),
         )
         assert attrs == {
+            "AREA_OR_POINT": "Area",
             "add_offset": 0.0,
             "scale_factor": 1.0,
         }
@@ -418,6 +428,8 @@ def create_tmp_geotiff(
         ) as s:
             for attr, val in additional_attrs.items():
                 setattr(s, attr, val)
+            for band in range(1, nz + 1):
+                s.update_tags(band, BAND=band)
             s.write(data, **write_kwargs)
             dx, dy = s.res[0], -s.res[1]
             tt = s.transform
@@ -472,6 +484,43 @@ def test_utm():
             assert "y" not in rioda.coords
 
 
+def test_band_as_variable(open_rasterio, tmp_path):
+    test_raster = tmp_path / "test.tif"
+
+    with create_tmp_geotiff() as (tmp_file, expected):
+        with open_rasterio(
+            tmp_file, band_as_variable=True, mask_and_scale=False
+        ) as riods:
+
+            def _check_raster(raster_ds):
+                for band in (1, 2, 3):
+                    band_name = f"band_{band}"
+                    assert_allclose(
+                        raster_ds[band_name], expected.sel(band=band).drop("band")
+                    )
+                    assert raster_ds[band_name].attrs["BAND"] == band
+                    assert raster_ds[band_name].attrs["scale_factor"] == 1.0
+                    assert raster_ds[band_name].attrs["add_offset"] == 0.0
+                    assert raster_ds[band_name].attrs["long_name"] == f"d{band}"
+                    assert raster_ds[band_name].attrs["units"] == f"u{band}"
+                    assert raster_ds[band_name].rio.crs == expected.rio.crs
+                    assert_array_equal(
+                        raster_ds[band_name].rio.resolution(), expected.rio.resolution()
+                    )
+                    assert isinstance(
+                        raster_ds[band_name].rio._cached_transform(), Affine
+                    )
+                    assert raster_ds[band_name].rio.nodata is None
+
+            _check_raster(riods)
+            # test roundtrip
+            riods.rio.to_raster(test_raster)
+            with open_rasterio(
+                test_raster, band_as_variable=True, mask_and_scale=False
+            ) as riods_round:
+                _check_raster(riods_round)
+
+
 def test_platecarree():
     with create_tmp_geotiff(
         8,
@@ -818,7 +867,7 @@ def test_rasterio_environment():
 
 
 @pytest.mark.xfail(
-    rasterio.__version__ == "1.1.1",
+    RASTERIO_VERSION == version.parse("1.1.1"),
     reason="https://github.com/mapbox/rasterio/issues/1833",
 )
 def test_rasterio_vrt():
@@ -943,6 +992,83 @@ def test_rasterio_vrt_gcps(tmp_path):
                 )
 
 
+@pytest.mark.skipif(
+    not RASTERIO_GE_13,
+    reason="warp options support added in 1.3.0",
+)
+def test_rasterio_vrt_warp_extras(tmp_path):
+    tiffname = tmp_path / "test.tif"
+    src_gcps = [
+        GroundControlPoint(
+            row=2015.0,
+            col=0.0,
+            x=100.61835695597287,
+            y=-0.19173548698662005,
+            z=685.0004720482975,
+            id="22",
+            info="",
+        ),
+        GroundControlPoint(
+            row=8060.0,
+            col=18990.0,
+            x=98.84559009470779,
+            y=-0.3783665839371584,
+            z=-0.00012378208339214325,
+            id="100",
+            info="",
+        ),
+        GroundControlPoint(
+            row=20150.0,
+            col=7596.0,
+            x=99.61705472917613,
+            y=-1.6823719472066612,
+            z=-7.35744833946228e-08,
+            id="217",
+            info="",
+        ),
+        GroundControlPoint(
+            row=22165.0,
+            col=22788.0,
+            x=98.2441590762303,
+            y=-1.5732331941915954,
+            z=-4.936009645462036e-08,
+            id="250",
+            info="",
+        ),
+        GroundControlPoint(
+            row=12090.0,
+            col=17724.0,
+            x=98.88075494473509,
+            y=-0.7646707270388453,
+            z=-0.0003558387979865074,
+            id="141",
+            info="",
+        ),
+    ]
+
+    crs = CRS.from_epsg(4326)
+
+    with rasterio.open(
+        tiffname,
+        mode="w",
+        height=23063,
+        width=25313,
+        count=1,
+        dtype=np.uint8,
+        driver="GTiff",
+    ) as source:
+        source.gcps = (src_gcps, crs)
+
+    with rasterio.open(tiffname) as src:
+        warp_extras = {"SRC_METHOD": "GCP_TPS"}
+        with rasterio.vrt.WarpedVRT(
+            src,
+            **warp_extras,
+        ) as vrt:
+            assert vrt.crs == "EPSG:4326"
+            assert vrt.shape == (28286, 29338)
+
+
 @cint_skip
 def test_rasterio_vrt_gcps__data_exists():
     # https://github.com/corteva/rioxarray/issues/515
@@ -952,50 +1078,48 @@ def test_rasterio_vrt_gcps__data_exists():
         # NOTE: Eventually src_crs will not need to be provided
         # https://github.com/mapbox/rasterio/pull/2193
         with rasterio.vrt.WarpedVRT(src, src_crs=crs) as vrt:
-            rds = rioxarray.open_rasterio(vrt)
-            assert rds.values.any()
+            with rioxarray.open_rasterio(vrt) as rds:
+                assert rds.values.any()
 
 
 @pytest.mark.parametrize("lock", [True, False])
 def test_open_cog(lock):
     cog_file = os.path.join(TEST_INPUT_DATA_DIR, "cog.tif")
-    rdsm = rioxarray.open_rasterio(cog_file)
-    assert rdsm.shape == (1, 500, 500)
-    rdso = rioxarray.open_rasterio(cog_file, lock=lock, overview_level=0)
-    assert rdso.shape == (1, 250, 250)
+    with rioxarray.open_rasterio(cog_file) as rdsm:
+        assert rdsm.shape == (1, 500, 500)
+    with rioxarray.open_rasterio(cog_file, lock=lock, overview_level=0) as rdso:
+        assert rdso.shape == (1, 250, 250)
 
 
 def test_mask_and_scale(open_rasterio):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "tmmx_20190121.nc")
-    with pytest.warns(SerializationWarning):
-        with open_rasterio(test_file, mask_and_scale=True) as rds:
-            assert np.nanmin(rds.air_temperature.values) == np.float32(248.7)
-            assert np.nanmax(rds.air_temperature.values) == np.float32(302.1)
-            test_encoding = dict(rds.air_temperature.encoding)
-            source = test_encoding.pop("source")
-            assert source.startswith("netcdf:") and source.endswith(
-                "tmmx_20190121.nc:air_temperature"
-            )
-            assert test_encoding == {
-                "_Unsigned": "true",
-                "add_offset": 220.0,
-                "scale_factor": 0.1,
-                "_FillValue": 32767.0,
-                "missing_value": 32767,
-                "grid_mapping": "crs",
-                "dtype": "uint16",
-                "rasterio_dtype": "uint16",
-            }
-            attrs = rds.air_temperature.attrs
-            assert attrs == {
-                "coordinates": "day",
-                "coordinate_system": "WGS84,EPSG:4326",
-                "description": "Daily Maximum Temperature",
-                "dimensions": "lon lat time",
-                "long_name": "tmmx",
-                "standard_name": "tmmx",
-                "units": "K",
-            }
+    with open_rasterio(test_file, mask_and_scale=True) as rds:
+        assert np.nanmin(rds.air_temperature.values) == np.float32(248.7)
+        assert np.nanmax(rds.air_temperature.values) == np.float32(302.1)
+        test_encoding = dict(rds.air_temperature.encoding)
+        source = test_encoding.pop("source")
+        assert source.startswith("netcdf:") and source.endswith(
+            "tmmx_20190121.nc:air_temperature"
+        )
+        assert test_encoding == {
+            "_Unsigned": "true",
+            "add_offset": 220.0,
+            "scale_factor": 0.1,
+            "_FillValue": 32767.0,
+            "missing_value": 32767,
+            "grid_mapping": "crs",
+            "dtype": "uint16",
+            "rasterio_dtype": "uint16",
+        }
+        attrs = rds.air_temperature.attrs
+        assert attrs == {
+            "coordinates": "day",
+            "description": "Daily Maximum Temperature",
+            "dimensions": "lon lat time",
+            "long_name": "tmmx",
+            "standard_name": "tmmx",
+            "units": "K",
+        }
 
 
 def test_no_mask_and_scale(open_rasterio):
@@ -1023,7 +1147,6 @@ def test_no_mask_and_scale(open_rasterio):
             "_Unsigned": "true",
             "add_offset": 220.0,
             "coordinates": "day",
-            "coordinate_system": "WGS84,EPSG:4326",
             "description": "Daily Maximum Temperature",
             "dimensions": "lon lat time",
             "long_name": "tmmx",
@@ -1033,19 +1156,71 @@ def test_no_mask_and_scale(open_rasterio):
         }
 
 
+def test_mask_and_scale__select_band_values(open_rasterio, tmp_path):
+    # https://github.com/corteva/rioxarray/issues/580
+    output_nc = tmp_path / "test.nc"
+    data = np.hypot(
+        *np.meshgrid(np.linspace(-100, 500, 50), np.linspace(-150, 700, 60))
+    )
+    xds = xr.Dataset(
+        {"var": (["y", "x"], data)},
+        coords={"x": np.linspace(40, 51, 50), "y": np.linspace(55, 62, 60)},
+    )
+    xds.rio.write_crs(4326, inplace=True)
+    xds.rio.write_coordinate_system(inplace=True)
+    encoding = {"var": {"scale_factor": 0.01, "_FillValue": -9999, "dtype": "int32"}}
+    xds.to_netcdf(output_nc, encoding=encoding)
+    with open_rasterio(output_nc) as rds:
+        if isinstance(rds, xr.Dataset):
+            rds = rds["var"]
+        assert_array_equal(rds.values[0], rds.isel(band=0).values)
+
+
 def test_mask_and_scale__to_raster(open_rasterio, tmp_path):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "tmmx_20190121.nc")
     tmp_output = tmp_path / "tmmx_20190121.tif"
-    with pytest.warns(SerializationWarning):
-        with open_rasterio(test_file, mask_and_scale=True) as rds:
-            rds.air_temperature.rio.to_raster(str(tmp_output))
-            with rasterio.open(str(tmp_output)) as riofh:
-                assert riofh.scales == (0.1,)
-                assert riofh.offsets == (220.0,)
-                assert riofh.nodata == 32767.0
-                data = riofh.read(1, masked=True)
-                assert data.min() == 287
-                assert data.max() == 821
+    with open_rasterio(test_file, mask_and_scale=True) as rds:
+        rds.air_temperature.rio.to_raster(str(tmp_output))
+        with rasterio.open(str(tmp_output)) as riofh:
+            assert riofh.scales == (0.1,)
+            assert riofh.offsets == (220.0,)
+            assert riofh.nodata == 32767.0
+            data = riofh.read(1, masked=True)
+            assert data.min() == 287
+            assert data.max() == 821
+
+
+def test_mask_and_scale__unicode(open_rasterio):
+    test_file = os.path.join(TEST_INPUT_DATA_DIR, "unicode.nc")
+    with open_rasterio(test_file, mask_and_scale=True) as rds:
+        assert np.nanmin(rds.LST.values) == np.float32(270.4925)
+        assert np.nanmax(rds.LST.values) == np.float32(276.6025)
+        test_encoding = dict(rds.LST.encoding)
+        assert test_encoding["_Unsigned"] == "true"
+        assert test_encoding["add_offset"] == 190
+        assert test_encoding["scale_factor"] == pytest.approx(0.0025)
+        assert test_encoding["_FillValue"] == 65535
+        expected_dtype = "int16"
+        if GDAL_GE_36:
+            # https://github.com/OSGeo/gdal/pull/6369
+            expected_dtype = "uint16"
+        assert test_encoding["dtype"] == expected_dtype
+        assert test_encoding["rasterio_dtype"] == expected_dtype
+
+
+def test_mask_and_scale__unicode__to_raster(open_rasterio, tmp_path):
+    tmp_output = tmp_path / "unicode.tif"
+    test_file = os.path.join(TEST_INPUT_DATA_DIR, "unicode.nc")
+    with open_rasterio(test_file, mask_and_scale=True) as rds:
+        rds.LST.rio.to_raster(str(tmp_output))
+        with rasterio.open(str(tmp_output)) as riofh:
+            assert riofh.scales == (pytest.approx(0.0025),)
+            assert riofh.offsets == (190,)
+            assert riofh.nodata == 65535
+            data = riofh.read(1, masked=True)
+            assert data.min() == 32197
+            assert data.max() == 34641
+            assert riofh.dtypes == ("uint16",)
 
 
 def test_notgeoreferenced_warning(open_rasterio):
@@ -1134,23 +1309,25 @@ def test_non_rectilinear():
 
 def test_non_rectilinear__load_coords(open_rasterio):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "2d_test.tif")
-    xds = open_rasterio(test_file)
-    assert xds.rio.shape == (10, 10)
-    with rasterio.open(test_file) as rds:
-        assert rds.transform == xds.rio.transform()
-        for xi, yi in itertools.product(range(rds.width), range(rds.height)):
-            subset = xds.isel(x=xi, y=yi)
-            assert_almost_equal(rds.xy(yi, xi), (subset.xc.item(), subset.yc.item()))
+    with open_rasterio(test_file) as xds:
+        assert xds.rio.shape == (10, 10)
+        with rasterio.open(test_file) as rds:
+            assert rds.transform == xds.rio.transform()
+            for xi, yi in itertools.product(range(rds.width), range(rds.height)):
+                subset = xds.isel(x=xi, y=yi)
+                assert_almost_equal(
+                    rds.xy(yi, xi), (subset.xc.item(), subset.yc.item())
+                )
 
 
 def test_non_rectilinear__skip_parse_coordinates(open_rasterio):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "2d_test.tif")
-    xds = open_rasterio(test_file, parse_coordinates=False)
-    assert "xc" not in xds.coords
-    assert "yc" not in xds.coords
-    assert xds.rio.shape == (10, 10)
-    with rasterio.open(test_file) as rds:
-        assert rds.transform == xds.rio.transform()
+    with open_rasterio(test_file, parse_coordinates=False) as xds:
+        assert "xc" not in xds.coords
+        assert "yc" not in xds.coords
+        assert xds.rio.shape == (10, 10)
+        with rasterio.open(test_file) as rds:
+            assert rds.transform == xds.rio.transform()
 
 
 def test_rotation_affine():
@@ -1178,14 +1355,14 @@ def test_rotation_affine():
 @pytest.mark.parametrize("dtype", [None, "complex_int16"])
 def test_cint16_dtype(dtype, tmp_path):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "cint16.tif")
-    xds = rioxarray.open_rasterio(test_file)
-    assert xds.rio.shape == (100, 100)
-    assert xds.dtype == "complex64"
-    assert xds.encoding["rasterio_dtype"] == "complex_int16"
+    with rioxarray.open_rasterio(test_file) as xds:
+        assert xds.rio.shape == (100, 100)
+        assert xds.dtype == "complex64"
+        assert xds.encoding["rasterio_dtype"] == "complex_int16"
 
-    tmp_output = tmp_path / "tmp_cint16.tif"
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype=dtype)
+        tmp_output = tmp_path / "tmp_cint16.tif"
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype=dtype)
     with rasterio.open(str(tmp_output)) as riofh:
         data = riofh.read()
         assert "complex_int16" in riofh.dtypes
@@ -1193,25 +1370,25 @@ def test_cint16_dtype(dtype, tmp_path):
 
 
 @pytest.mark.skipif(
-    rasterio.__version__ < "1.2.5",
+    not RASTERIO_GE_125,
     reason="https://github.com/mapbox/rasterio/issues/2206",
 )
 def test_cint16_dtype_nodata(tmp_path):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "cint16.tif")
-    xds = rioxarray.open_rasterio(test_file)
-    assert xds.rio.nodata == 0
+    with rioxarray.open_rasterio(test_file) as xds:
+        assert xds.rio.nodata == 0
 
-    tmp_output = tmp_path / "tmp_cint16.tif"
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype="complex_int16")
-    with rasterio.open(str(tmp_output)) as riofh:
-        assert riofh.nodata == 0
+        tmp_output = tmp_path / "tmp_cint16.tif"
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype="complex_int16")
+        with rasterio.open(str(tmp_output)) as riofh:
+            assert riofh.nodata == 0
 
-    # Assign nodata=None
-    tmp_output = tmp_path / "tmp_cint16_nodata.tif"
-    xds.rio.write_nodata(None, inplace=True)
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype="complex_int16")
+        # Assign nodata=None
+        tmp_output = tmp_path / "tmp_cint16_nodata.tif"
+        xds.rio.write_nodata(None, inplace=True)
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype="complex_int16")
     with rasterio.open(str(tmp_output)) as riofh:
         assert riofh.nodata is None
 
@@ -1220,15 +1397,15 @@ def test_cint16_dtype_nodata(tmp_path):
 @pytest.mark.parametrize("dtype", [None, "complex_int16"])
 def test_cint16_dtype_masked(dtype, tmp_path):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "cint16.tif")
-    xds = rioxarray.open_rasterio(test_file, masked=True)
-    assert xds.rio.shape == (100, 100)
-    assert xds.dtype == "complex64"
-    assert xds.rio.encoded_nodata == 0
-    assert np.isnan(xds.rio.nodata)
+    with rioxarray.open_rasterio(test_file, masked=True) as xds:
+        assert xds.rio.shape == (100, 100)
+        assert xds.dtype == "complex64"
+        assert xds.rio.encoded_nodata == 0
+        assert np.isnan(xds.rio.nodata)
 
-    tmp_output = tmp_path / "tmp_cint16.tif"
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype=dtype)
+        tmp_output = tmp_path / "tmp_cint16.tif"
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype=dtype)
     with rasterio.open(str(tmp_output)) as riofh:
         data = riofh.read()
         assert "complex_int16" in riofh.dtypes
@@ -1239,20 +1416,20 @@ def test_cint16_dtype_masked(dtype, tmp_path):
 @cint_skip
 def test_cint16_promote_dtype(tmp_path):
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "cint16.tif")
-    xds = rioxarray.open_rasterio(test_file)
+    with rioxarray.open_rasterio(test_file) as xds:
 
-    tmp_output = tmp_path / "tmp_cfloat64.tif"
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype="complex64")
-    with rasterio.open(str(tmp_output)) as riofh:
-        data = riofh.read()
-        assert "complex64" in riofh.dtypes
-        assert riofh.nodata == 0
-        assert data.dtype == "complex64"
+        tmp_output = tmp_path / "tmp_cfloat64.tif"
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype="complex64")
+        with rasterio.open(str(tmp_output)) as riofh:
+            data = riofh.read()
+            assert "complex64" in riofh.dtypes
+            assert riofh.nodata == 0
+            assert data.dtype == "complex64"
 
-    tmp_output = tmp_path / "tmp_cfloat128.tif"
-    with pytest.warns(NotGeoreferencedWarning):
-        xds.rio.to_raster(str(tmp_output), dtype="complex128")
+        tmp_output = tmp_path / "tmp_cfloat128.tif"
+        with pytest.warns(NotGeoreferencedWarning):
+            xds.rio.to_raster(str(tmp_output), dtype="complex128")
     with rasterio.open(str(tmp_output)) as riofh:
         data = riofh.read()
         assert "complex128" in riofh.dtypes
@@ -1316,13 +1493,15 @@ def test_read_file_handle_with_dask():
     with open(
         os.path.join(TEST_COMPARE_DATA_DIR, "small_dem_3m_merged.tif"), "rb"
     ) as src:
-        rioxarray.open_rasterio(src, chunks=2048)
+        with rioxarray.open_rasterio(src, chunks=2048):
+            pass
 
 
 @cint_skip
 def test_read_cint16_with_dask():
     test_file = os.path.join(TEST_INPUT_DATA_DIR, "cint16.tif")
-    rioxarray.open_rasterio(test_file, chunks=True)
+    with rioxarray.open_rasterio(test_file, chunks=True):
+        pass
 
 
 @python_vsi_skip
